RecapFu = AceLibrary("AceAddon-2.0"):new("AceConsole-2.0", "AceDB-2.0", "AceDebug-2.0", "FuBarPlugin-2.0", "AceEvent-2.0")

local L = AceLibrary("AceLocale-2.2"):new("RecapFu")
local tablet = AceLibrary("Tablet-2.0")
local crayon = AceLibrary("Crayon-2.0")
local dewdrop = AceLibrary("Dewdrop-2.0")

local TYPES = {
	a = {var = "DPS", text = "Your DPS"},
	b = {var = "DPS_IN", text = "Total DPS In"},
	c = {var = "DPS_OUT", text = "Total DPS Out"},
	d = {var = "HEALING", text = "Healing"},
	e = {var = "OVERHEAL", text = "Overhealing"},
	f = {var = "MAXHIT", text = "Max hit"},
}

RecapFu.version = "2.0." .. string.sub("$Revision: 22518 $", 12, -3)
RecapFu.date = string.sub("$Date: 2006-12-29 01:22:26 -0500 (Fri, 29 Dec 2006) $", 8, 17)
RecapFu.hasIcon = "Interface\\AddOns\\Recap\\Recap-Status"
RecapFu.hasNoColor = true
RecapFu.independentProfile = true

RecapFu:RegisterChatCommand({"/recapfu", "/fubar_recapfu"})

RecapFu:RegisterDB("RecapFuDB")
RecapFu:RegisterDefaults('profile', {
	showlabel = true,
   	DPS = true,
})

function RecapFu:OnInitialize()
end

function RecapFu:OnEnable()
	self.text = {}
	self:ScheduleRepeatingEvent(self.Update, 1, self)
end

function RecapFu:OnDisable()
	self.state = "n/a"
	self.yourdps = "n/a"
	self.dpsin = "n/a"
	self.dpsout = "n/a"
	self.healing = "n/a"
	self.overheal = "n/a"
	self.maxhit = "n/a"
	self.totaldps = "n/a"
	self.totaldpsin = "n/a"
	self.viewtype = "n/a"
end

function RecapFu:OnClick()
	RecapFrame_Toggle()
end

function RecapFu:UpdateData()
	self:Debug("Update Data")
	if recap.Opt then
		if recap.Opt.State then
			self.state = recap.Opt.State.value or "n/a"
		else
			self.state = "n/a"
		end
		if recap.Opt.View and recap_temp.Local.LastAll then
			self.viewtype = recap_temp.Local.LastAll[recap.Opt.View.value] or "n/a"
		else
			self.viewtype = "n/a"
		end
	else
		self.state = "n/a"
		self.viewtype = "n/a"
	end
	self.yourdps = RecapMinYourDPS_Text:GetText() or "n/a"
	self.dpsin = RecapMinDPSIn_Text:GetText() or "n/a"
	self.dpsout = RecapMinDPSOut_Text:GetText() or "n/a"
	self.healing = RecapTotals_Heal:GetText() or 0
	self.overheal = RecapTotals_Over:GetText() or 0
	self.maxhit = RecapTotals_MaxHit:GetText() or "n/a"
	self.totaldps = RecapTotals_DPS:GetText() or "n/a"
	self.totaldpsin = RecapTotals_DPSIn:GetText() or "n/a"
	
	self.text.DPS = crayon:White(self.yourdps)
	self.text.DPS_IN = crayon:Red(self.dpsin)
	self.text.DPS_OUT = crayon:Green(self.dpsout)
	self.text.HEALING = crayon:Colorize("00ffff", self.healing)
	self.text.OVERHEAL = crayon:Colorize("00ffff", self.overheal)
	self.text.MAXHIT = crayon:Orange(self.maxhit)
end

function RecapFu:UpdateText()
	self:Debug("Update Text")
	-- format the colored status bubble
	if (self:IsIconShown()) then
		if self.state=="Idle" then
			self.iconFrame:SetVertexColor(.5,.5,.5)
		elseif self.state=="Active" then
			self.iconFrame:SetVertexColor(0,1,0)
		elseif self.state=="Stopped" then
			self.iconFrame:SetVertexColor(1,0,0)
		end
	end

	local t = {}
	table.insert(t, self.db.profile.showlabel and L["DPS"]..": " or "")
	
	for i,e in self:pairsByKeys(TYPES) do
		if self.db.profile[e.var] then
			table.insert(t, self.text[e.var])
		end
	end
	
	self:SetText(table.concat(t, " "))
end

function RecapFu:OnTooltipUpdate()
	self:Debug("Update Tooltip")
	local cat = tablet:AddCategory(
		'columns', 2,
		'child_textR', 1,
		'child_textG', 1,
		'child_textB', 0,
		'child_text2R', 1,
		'child_text2G', 1,
		'child_text2B', 1
	)
	cat:AddLine('text', L["State"], 'text2', self.state)
	cat:AddLine('text', L["View Type"], 'text2', self.viewtype)
	
	local cat = tablet:AddCategory(
		'columns', 2,
		'child_textR', 1,
		'child_textG', 1,
		'child_textB', 0
	)
	for i,e in self:pairsByKeys(TYPES) do
		cat:AddLine('text', L[e.text], 'text2', self.text[e.var])
	end
	tablet:SetHint(L["Left-click to toggle Recap window"])
end

function RecapFu:OnMenuRequest(level, value)
	if level == 1 then
		dewdrop:AddLine(
			'text', L["Display"],
			'hasArrow', true,
			'value', "display"
		)
		dewdrop:AddLine()
		if (recap and recap.Opt and recap.Opt.Paused.value == true) then
			dewdrop:AddLine(
				'text', L["Resume Monitoring"],
				'func', function() 
					Recap_OnClick("Pause")
					self:Update()
				end,
				'closeWhenClicked', true
			)
		else
			dewdrop:AddLine(
				'text', L["Pause Monitoring"],
				'func', function()
					Recap_OnClick("Pause")
					self:Update()
				end,
				'closeWhenClicked', true
			)
		end
		dewdrop:AddLine()
	elseif level == 2 then
		if value == "display" then
			dewdrop:AddLine(
				'text', L["Label"],
				'func', function()
					self:ToggleOption("showlabel", true)
				end,
				'checked', self.db.profile.showlabel
			)
			for i,e in self:pairsByKeys(TYPES) do
				local my_e = e
				dewdrop:AddLine(
					'text', L[my_e.text],
					'func', function() self:ToggleOption(my_e.var, true) end,
					'checked', self.db.profile[my_e.var]
				)
			end
		end
	end
end

function RecapFu:ToggleOption(var, doUpdate)
	self.db.profile[var] = not self.db.profile[var]
	if doUpdate then
		self:Update()
	end
	return self.db.profile[var]
end

function RecapFu:pairsByKeys (t, f)
--taken from an example in the Programming in Lua book
	local a = {}
	for n in pairs(t) do
		table.insert(a, n)
	end
	table.sort(a, f)
	local i = 0      -- iterator variable
	local iter = function ()   -- iterator function
		i = i + 1
		if a[i] == nil then
			return nil
		else
			return a[i], t[a[i]]
		end
	end
	return iter
end

-- Simulate Titan Panel plugin
-- Necessary or updates to dps data on the plugin
-- may not occur until after combat ends
function TitanPanelRecap_Update(state,yourdps,dpsin,dpsout)
	RecapFu.state = state
	RecapFu.yourdps = yourdps
	RecapFu.dpsin = dpsin
	RecapFu.dpsout = dpsout
	RecapFu:Update()
end